@@ -6,68 +6,65 @@
 sparse_template <- function(mc, update.XX=FALSE, add.outer.R=FALSE) {
 
   if (mc[["type"]] == "block") {
-    Q <- mc$QT
+    Q <- mc[["QT"]]
     keep.kp <- FALSE
   } else {
     # build a Kronecker product template
-    if (mc$var == "unstructured") {
-      Qv <- rWishart(1L, df=mc$prior$df, Sigma=diag(mc$q0))[,,1L]
-    } else if (mc$var == "scalar" && !is.null(mc$Q0)) {
-      Qv <- scale_mat(mc$Q0, runif(if (mc$usePX && mc$PX$vector) mc$q0 else 1L, 0.25, 0.75))
-    } else if (mc$var == "diagonal" || (mc$usePX && mc$PX$vector)) {
-      Qv <- runif(mc$q0, 0.25, 0.75)
-    } else {  # scalar var, scalar or no PX
-      Qv <- runif(1L, 0.25, 0.75)
-    }
-    if (mc$gl) {
+    Qv <- mc$generate_Qv()
+    if (mc[["gl"]]) {
       QA <- mc$glp$QA.ext
-    } else if (mc$Leroux_update) {
-      QA <- mc$mat_sum_Leroux(mc$QA, mc$idL, w2=runif(1L, 0.25, 0.75))
+    } else if (mc$strucA$update.Q) {
+      QA <- mc$strucA$update_Q(mc[["QA"]], runif(1L, 0.25, 0.75))
     } else if (is.null(mc$priorA)) {
-      QA <- mc$QA
+      QA <- if (is.null(mc[["AR1.inferred"]])) mc[["QA"]] else mc$QA.template$QA0.5
     } else {
-      QA <- crossprod_sym(mc$DA, runif(mc$lD, 0.75, 1.25))
+      if (is.null(mc[["AR1.inferred"]]))
+        QA <- crossprod_sym(mc$DA, runif(mc$lD, 0.75, 1.25))
+      else
+        QA <- crossprod_sym(mc$DA.template$DA0.5, runif(mc$lD, 0.75, 1.25))
     }
 
-    kron_prod <- build_kron(QA, Qv, q2=mc$q0, M1.fixed = is.null(mc$priorA) && !mc$Leroux_update)
+    kron_prod <- build_kron(QA, Qv, q2=mc[["q0"]],
+      M1.fixed = is.null(mc[["priorA"]]) && !mc$strucA$update.Q && is.null(mc[["AR1.inferred"]])
+    )
     Q <- kron_prod(QA, Qv)
-    if (mc$in_block) assign("Q", Q, envir=mc)  # only for single-time use in create_mc_block
+    if (mc[["in_block"]]) assign("Q", Q, envir=mc)  # only for single-time use in create_mc_block
     keep.kp <- TRUE
   }
 
-  if (mc[["type"]] == "block" || !mc$in_block) {
-    if (mc[["type"]] == "gen" && mc$gl) {
-      XX <- mc$glp$XX.ext
-      R <- mc$glp$R
+  if (mc[["type"]] == "block" || !mc[["in_block"]]) {
+    if (mc[["type"]] == "gen" && mc[["gl"]]) {
+      XX <- mc$glp[["XX.ext"]]
+      R <- mc$glp[["R"]]
     } else {
-      XX <- mc$XX
-      R <- mc$R
+      XX <- mc[["XX"]]
+      R <- mc[["R"]]
     }
 
-    if (mc$e$control$expanded.cMVN.sampler) {
+    if (mc[["e"]]$control$expanded.cMVN.sampler) {
       if (update.XX) {
-        mat_sum <- make_mat_sum(M1=XX, M2=Q)
+        mat_sum <- make_mat_sum(M1=XX, M2=Q, force.sparse=TRUE)
         XX_Q <- mat_sum(XX, Q)
       } else {
-        mat_sum <- make_mat_sum(M0=XX, M1=Q)
+        mat_sum <- make_mat_sum(M0=XX, M1=Q, force.sparse=TRUE)
         XX_Q <- mat_sum(Q)
       }
-      mc$MVNsampler <- create_cMVN_sampler(
+      mc$MVNsampler <- create_block_cMVN_sampler(
         mbs=mc[["mcs"]], X=mc[["X"]], Q=XX_Q,
-        R=R, r=mc$r,
-        sampler=mc$e,
-        name=mc$name, chol.control=mc$e$control$chol.control
+        R=R, r=mc[["r"]],
+        sampler=mc[["e"]],
+        name=mc[["name"]], chol.control=mc[["e"]]$control$chol.control
       )
     } else {
       mc$MVNsampler <- NULL
       R0 <- R
       if (is.null(add.outer.R)) {
         # unresolved singularities usually only occur when at least two components are improper
-        add.outer.R <- mc[["type"]] == "block" && sum(!sapply(mc$mcs, is_proper_GMRF)) >= 2L
+        add.outer.R <- mc[["type"]] == "block" && sum(!b_apply(mc[["mcs"]], `[[`, "is.proper")) >= 2L
         if (add.outer.R && !is.null(R)) {
           # it should not always be necessary to use the full matrix R
           # leave out the largest of the individual model components' contribution:
-          cols <- sapply(mc$mcs, function(x) if (is.null(x$R)) 0L else ncol(x$R))
+          cols <- i_apply(mc[["mcs"]], function(x) if (is.null(x[["R"]])) 0L else ncol(x[["R"]]))
           cols <- cols[cols != 0L]
           if (length(cols) > 1L) {
             inds <- c(0L, cumsum(cols))
@@ -87,9 +84,9 @@ sparse_template <- function(mc, update.XX=FALSE, add.outer.R=FALSE) {
         tryCatch(
           suppressWarnings(
             mc$MVNsampler <- create_TMVN_sampler(
-              Q=XX_Q, update.Q=TRUE, name=mc$name,
-              R=R, r=mc$r, S=mc$S, s=mc$s,
-              chol.control=mc$e$control$chol.control
+              Q=XX_Q, update.Q=TRUE, name=mc[["name"]],
+              R=R, r=mc[["r"]], S=mc[["S"]], s=mc[["s"]],
+              chol.control=mc[["e"]]$control$chol.control
             )
           ),
           error = function(e) {
@@ -104,11 +101,11 @@ sparse_template <- function(mc, update.XX=FALSE, add.outer.R=FALSE) {
             #   and perhaps use weights to further reduce the condition number
             if (is.null(R)) {
               if (mc[["type"]] == "block") {
-                noninf <- sapply(mc$mcs, function(x) isFALSE(x$informative.prior))
+                noninf <- b_apply(mc[["mcs"]], function(x) isFALSE(x$informative.prior))
                 if (any(noninf))
                   warn("error in setting up a multivariate normal sampler, possibly due to ",
                        "a singular precision matrix; you may try to replace the non-informative prior ",
-                       "of model component(s) '", paste(names(noninf)[noninf], collapse="', '"),
+                       "of model component(s) '", paste0(names(noninf)[noninf], collapse="', '"),
                        "' by a (weakly) informative prior")
               }
               stop(e)
@@ -134,20 +131,20 @@ sparse_template <- function(mc, update.XX=FALSE, add.outer.R=FALSE) {
         tryCatch(
           suppressWarnings(
             mc$MVNsampler <- create_TMVN_sampler(
-              Q=XX_Q, update.Q=TRUE, name=mc$name,
-              R=R, r=mc$r, S=mc$S, s=mc$s,
-              chol.control=mc$e$control$chol.control
+              Q=XX_Q, update.Q=TRUE, name=mc[["name"]],
+              R=R, r=mc[["r"]], S=mc[["S"]], s=mc[["s"]],
+              chol.control=mc[["e"]]$control$chol.control
             )
           ),
           error = function(e) {
             if (mc[["type"]] == "block") {
-              noninf <- sapply(mc$mcs, function(x) isFALSE(x$informative.prior) ||
-                                 (x[["type"]] == "gen" && x$gl && isFALSE(x$glp$informative.prior))
+              noninf <- b_apply(mc[["mcs"]], function(x)
+                isFALSE(x$informative.prior) || (x[["type"]] == "gen" && x[["gl"]] && isFALSE(x$glp$informative.prior))
               )
               if (any(noninf))
                 warn("error in setting up a multivariate normal sampler, possibly due to ",
                      "a singular precision matrix; you may try to replace the non-informative prior ",
-                     "of model component(s) '", paste(names(noninf)[noninf], collapse="', '"),
+                     "of model component(s) '", paste0(names(noninf)[noninf], collapse="', '"),
                      "' by a (weakly) informative prior")
             }
             stop(e)
@@ -157,10 +154,10 @@ sparse_template <- function(mc, update.XX=FALSE, add.outer.R=FALSE) {
     }
 
     keep.ms <- TRUE
-    if (mc[["type"]] == "gen" && mc$unit_Q) {  # unit QA, scalar Qv (with unit Q0), and no constraints
-      if (isUnitDiag(XX)) {
+    if (mc[["type"]] == "gen" && mc[["unit_Q"]]) {  # unit QA, scalar Qv (with unit Q0), and no constraints
+      if (is_unit_diag(XX)) {
         # TODO cleaner and more efficient handling of unit XX case (unit_Q + unit XX --> scalar update)
-        XX.expanded <- expandUnitDiag(XX)
+        if (is_unit_ddi(XX.expanded)) XX.expanded <- expand_unit_ddi(XX)
         mc$update <- function(XX, QA, Qv, w) list(Q=XX.expanded, Imult=Qv*w)
         keep.kp <- keep.ms <- FALSE
       } else {
@@ -175,8 +172,8 @@ sparse_template <- function(mc, update.XX=FALSE, add.outer.R=FALSE) {
         }
       }
     } else {
-      if (mc[["type"]] == "block" || mc$q0 == 1L) {  # scalar Qv, or block
-        if (mc[["type"]] != "block") keep.kp <- FALSE
+      if (mc[["type"]] == "block" || mc[["q0"]] == 1L) {  # scalar Qv, or block
+        if (mc[["type"]] != "block" && is.null(mc[["AR1.inferred"]])) keep.kp <- FALSE
         if (update.XX)
           mc$update <- function(XX, QA, Qv, w) list(Q=mat_sum(XX, QA, w2=Qv*w), Imult=0)
         else
