@@ -16,7 +16,7 @@ test_that("Cholesky for ddiMatrix works", {
   expect_equal(ch$solve(y1), y1/M@x)
   expect_equal(ch$solve(y2), Diagonal(x=1/M@x) %*% y2)
   expect_equal(ch$solve(y0, system="Lt"), y0/sqrt(M@x))
-  expect_equal(ch$solve(y0, system="Lt", systemP=TRUE), y0/sqrt(M@x))
+  expect_equal(ch$solve(y0, system="Lt"), y0/sqrt(M@x))
   expect_equal(ch$solve(y0, system="L"), y0/sqrt(M@x))
   expect_equal(ch$solve(y1, system="Lt"), y1/sqrt(M@x))
   expect_equal(ch$solve(y2, system="Lt"), Diagonal(x=1/sqrt(M@x)) %*% y2)
@@ -59,8 +59,6 @@ test_that("Cholesky for dsCMatrix works", {
   expect_equal(ch$solve(y1, system="Lt"), array(solve(cholM, y1, system="Lt")@x, dim(y1)))
   expect_equal(ch$solve(y2, system="Lt"), solve(cholM, y2, system="Lt"))
   expect_equal(ch$solve(y0, system="L"), as.vector(solve(cholM, y0, system="L")))
-  # NB disable following test; currently cannot control cholmod ordering
-  #expect_equal(ch$solve(y0, system="L", systemP=TRUE), as.vector(solve(cholM, y0, system="L")))
   expect_equal(ch$solve(y1, system="L"), array(solve(cholM, y1, system="L")@x, dim(y1)))
   expect_equal(ch$solve(y2, system="L"), solve(cholM, y2, system="L"))
   # with permutation
@@ -71,10 +69,10 @@ test_that("Cholesky for dsCMatrix works", {
   expect_equal(ch$solve(y0), as.vector(solve(cholM, y0)))
   expect_equal(ch$solve(y1), array(solve(cholM, y1)@x, dim(y1)))
   expect_equal(ch$solve(y2), solve(cholM, y2))
-  expect_equal(ch$solve(y0, system="Lt"), as.vector(solve(cholM, y0, system="Lt")))
-  expect_equal(ch$solve(y1, system="Lt"), array(solve(cholM, y1, system="Lt")@x, dim(y1)))
-  expect_equal(ch$solve(y2, system="Lt"), solve(cholM, y2, system="Lt"))
-  expect_equal(ch$solve(y0, system="L"), as.vector(solve(cholM, y0, system="L")))
-  expect_equal(ch$solve(y1, system="L"), array(solve(cholM, y1, system="L")@x, dim(y1)))
-  expect_equal(ch$solve(y2, system="L"), solve(cholM, y2, system="L"))
+  expect_equal(ch$solve(y0, system="Lt"), as.vector(solve(cholM, solve(cholM, y0, system="Lt"), system="Pt")))
+  expect_equal(ch$solve(y1, system="Lt"), array(solve(cholM, solve(cholM, y1, system="Lt"), system="Pt")@x, dim(y1)))
+  expect_equal(ch$solve(y2, system="Lt"), solve(cholM, solve(cholM, y2, system="Lt"), system="Pt"))
+  expect_equal(ch$solve(y0, system="L"), as.vector(solve(cholM, solve(cholM, y0, system="P"), system="L")))
+  expect_equal(ch$solve(y1, system="L"), array(solve(cholM, solve(cholM, y1, system="P"), system="L")@x, dim(y1)))
+  expect_equal(ch$solve(y2, system="L"), solve(cholM, solve(cholM, y2, system="P"), system="L"))
 })
